#
#  This module provides a playground-grid with a small figurine in it
#  like e.g. a PacMan or a Turtle. The figure can move around inside
#  the grid unless its path is blocked by black cells.
#
#  Example:
#  ------------------------------------------------------------------
#
#  from tjgrids import *
#  def onStep():
#      if getCell() == 4:
#          print "Goal reached!"
#          stop()   
#      elif canGoForward():
#          forward()
#      else:
#          right()
#
#  setCell(5, 6, 4)
#  start(onStep)
#
#  ------------------------------------------------------------------
#
#  (c) 2013, T. Kohn
#  www.tobiaskohn.ch/jython
#
#  Last updated: 03-DEC-2013
#
from tigerjython.utils import JythonGrid, JythonGridWindow, GridFigure
from time import sleep

_mainfigure = None
_maingrid = None
__mainwindow = None
__automode = False
__std_waittime = 0.125

def getFigure():
    global _mainfigure
    return _mainfigure

def getGrid():
    global _maingrid
    return _maingrid

def getGridWindow():
    global __mainwindow
    return __mainwindow

# Create the figure, window etc.
def __createGrid(showWindow = True):
    """Create the main window with the grid inside it."""
    global _maingrid, __mainwindow
    __mainwindow = JythonGridWindow()
    _maingrid = __mainwindow.grid()
    if showWindow:
        __mainwindow.show()
    
def __createFigure():
    """Create a figure which will move around within the grid."""
    global _mainfigure, _maingrid
    if _maingrid == None:
        __createGrid()
    _mainfigure = GridFigure(_maingrid)
    _maingrid.addFigure(_mainfigure)
    
def makeGrid(*args):
    global _mainfigure, _maingrid, __mainwindow
    __createGrid(False)
    if len(args) == 1:
        _maingrid.loadFromFile(args[0])
        fig = _maingrid.getFirstFigure()
        if fig != None:
            _mainfigure = fig
    elif len(args) == 2:
        _maingrid.setSize(args[0], args[1])
    elif len(args) == 3:
        _maingrid.setSize(args[0], args[1], args[2])
    __mainwindow.show()
    
def makeTurtle(*args):
    makeGrid(*args)
    setShape("turtle")
    __wait()
    
def saveToFile(filename):
    global _maingrid
    if _maingrid != None:
        _maingrid.saveToFile(filename)
    
# Move the figure. If necessary, create it first.
def __wait():
    global __automode, __std_waittime
    if not __automode:
        sleep(__std_waittime)

def forward(s = 1):
    global _mainfigure, __automode
    if _mainfigure == None:
        __createFigure()
    if _mainfigure != None:
        if not __automode:
            for i in xrange(s):
                _mainfigure.forward(1)
                __wait()
        else:
            _mainfigure.forward(s)

def back(s = 0):
    global _mainfigure
    if _mainfigure == None:
        __createFigure()
    if _mainfigure != None:
        _mainfigure.back(s)
        __wait()
        
def left(*args):
    global _mainfigure
    if _mainfigure == None:
        __createFigure()
    if _mainfigure != None:
        if len(args) == 1:
            angle = int((args[0] + 45) / 90)
            for i in range(angle):
                _mainfigure.turnLeft()
        else:
            _mainfigure.turnLeft()
        __wait()

def right(*args):
    global _mainfigure
    if _mainfigure == None:
        __createFigure()
    if _mainfigure != None:
        if len(args) == 1:
            angle = int((args[0] + 45) / 90)
            for i in range(angle):
                _mainfigure.turnRight()
        else:
            _mainfigure.turnRight()
        __wait()

def moveTo(x, y):
    global _mainfigure
    if _mainfigure == None:
        __createFigure()
    if _mainfigure != None:
        _mainfigure.moveTo(x, y)
        
def setPos(x, y):
    global _mainfigure
    if _mainfigure == None:
        __createFigure()
    if _mainfigure != None:
        _mainfigure.setPos(x, y)
        
def getX():
    global _mainfigure
    if _mainfigure != None:
        return _mainfigure.posX()
    else:
        return -1
        
def getY():
    global _mainfigure
    if _mainfigure != None:
        return _mainfigure.posY()
    else:
        return -1
        
def goForward():
    forward(1)
    
def goLeft():
    left()
    forward(1)
    
def goRight():
    right()
    forward(1)
        
# Receive the current values of cells around the figure
# or set the value of the cell below the figure.
def getCell(*args):
    global _mainfigure, _maingrid
    if len(args) == 0:
        if _mainfigure != None:
            return _mainfigure.getCell()
        else:
            return 0
    elif len(args) == 2:
        if _maingrid != None:
            _maingrid.getCellValue(args[0], args[1])
        else:
            return 0
    else:
        return 0
        
def isCell(*args):
    if len(args) == 1:
        return getCell() == args[0]
    if len(args) == 3:
        return getCell(args[0], args[1]) == args[2]
    return 0
        
def setCell(*args):
    global _mainfigure, _maingrid
    if len(args) == 0 or len(args) == 1:
        if len(args) == 1:
            value = args[0]
        else:
            value = 1
        if _mainfigure == None:
            __createFigure()
        if _mainfigure != None:
            _mainfigure.setCell(value)
    elif len(args) == 2 or len(args) == 3:
        if len(args) == 3:
            value = args[2]
        else:
            value = 1
        if _maingrid == None:
            __createGrid()
        if _maingrid != None:
            _maingrid.setCellValue(args[0], args[1], value)

def toggleCell():
    global _mainfigure
    if _mainfigure == None:
        __createFigure()
    if _mainfigure != None:
        _mainfigure.toggleCell()
        
def getCellForward():
    global _mainfigure
    if _mainfigure != None:
        return _mainfigure.getCellForward()
    else:
        return 0

def getCellLeft():
    global _mainfigure
    if _mainfigure != None:
        return _mainfigure.getCellLeft()
    else:
        return 0

def getCellRight():
    global _mainfigure
    if _mainfigure != None:
        return _mainfigure.getCellRight()
    else:
        return 0

def getCellBack():
    global _mainfigure
    if _mainfigure != None:
        return _mainfigure.getCellBack()
    else:
        return 0
        
def canGo(*args):
    global _mainfigure
    if _mainfigure != None:
        if len(args) == 0:
            return _mainfigure.canGo()
        elif len(args) == 2:
            return _mainfigure.canGo(args[0], args[1])
    else:
        return False
        
def canGoForward():
    global _mainfigure
    if _mainfigure != None:
        return _mainfigure.canGoForward()
    else:
        return False
    
def canGoLeft():
    global _mainfigure
    if _mainfigure != None:
        return _mainfigure.canGoLeft()
    else:
        return False
    
def canGoRight():
    global _mainfigure
    if _mainfigure != None:
        return _mainfigure.canGoRight()
    else:
        return False
    
def canGoBack():
    global _mainfigure
    if _mainfigure != None:
        return _mainfigure.canGoBack()
    else:
        return False
        
def canGoAnywhere():
    global _mainfigure
    if _mainfigure != None:
        return _mainfigure.canGoAnywhere()
    else:
        return False

# Show or hide the figure.
def show():
    global _mainfigure
    if _mainfigure == None:
        __createFigure()
    if _mainfigure != None:
        _mainfigure.show()
        
def hide():
    global _mainfigure
    if _mainfigure != None:
        _mainfigure.hide()
        
def setShape(s):
    global _mainfigure
    if _mainfigure == None:
        __createFigure()
    if _mainfigure != None:
        _mainfigure.setShape(s)

# Use "start(onStep)" to start a simple animation where
# the figure can act, based upon the given method.
__canGo = True
__waittime = 0.3

def speed(value):
    global __waittime, __std_waittime
    if value >= 1 and value <= 20:
        __waittime = int(40 / value) / 20
        __std_waittime = __waittime
    elif value == 0:
        __waittime = 0
        __std_waittime = 0

def start(onStep = None):
    global __canGo, _maingrid, __mainwindow, __automode
    if _maingrid == None:
        __createGrid()
    if onStep != None and _maingrid != None:
        __canGo = True
        __automode = True
        while __canGo and canGoAnywhere():
            if not _maingrid.isPaused():
                onStep()
            if not __mainwindow.visible():
                break
            sleep(__waittime)
    __automode = False

def startOnEnter(onStep = None):
    global _maingrid, _mainfigure
    if _maingrid == None:
        __createGrid()
    if _mainfigure == None:
        __createFigure()
    if _maingrid != None and onStep != None:
        _maingrid.waitForEnter()
        start(onStep)
            
def stop():
    global __canGo, __automode
    __canGo = False
    __automode = False

# Events

def setOnClick(handler):
    global _maingrid
    if _maingrid == None:
        __createGrid
    if _maingrid != None:
        _maingrid.setOnMouseClick(handler)

def setOnKey(handler):
    global _maingrid
    if _maingrid == None:
        __createGrid
    if _maingrid != None:
        _maingrid.setOnKeyTyped(handler)
